@@ -29,48 +29,114 @@
 #include "preferences.h"
 
 
+typedef struct {
+	GthBrowser *browser;
+	GFile      *parent;
+} NewFolderData;
+
+
 static void
-_gth_browser_create_new_folder (GthBrowser *browser,
-				GFile      *parent)
+new_fodler_data_free (NewFolderData *data)
 {
-	char   *folder_name;
-	GError *error = NULL;
-	GFile  *new_folder;
+	g_object_unref (data->parent);
+	g_free (data);
+}
 
-	folder_name = _gtk_request_dialog_run (GTK_WINDOW (browser),
-					       GTK_DIALOG_MODAL,
-					       _("Enter the folder name: "),
-				               "",
-					       1024,
-					       GTK_STOCK_CANCEL,
-					       _("C_reate"));
-	if (folder_name == NULL)
+
+static void
+new_folder_dialog_response_cb (GtkWidget *dialog,
+			       int        response_id,
+			       gpointer   user_data)
+{
+	NewFolderData *data = user_data;
+	char          *name;
+	GFile         *folder;
+	GError        *error = NULL;
+
+	if (response_id != GTK_RESPONSE_OK) {
+		new_fodler_data_free (data);
+		gtk_widget_destroy (dialog);
 		return;
+	}
 
-	new_folder = g_file_get_child_for_display_name (parent, folder_name, &error);
-	if ((new_folder != NULL) && (g_file_make_directory (new_folder, NULL, &error))) {
+	name = gth_request_dialog_get_normalized_text (GTH_REQUEST_DIALOG (dialog));
+	if (_g_utf8_all_spaces (name)) {
+		g_free (name);
+		gth_request_dialog_set_info_text (GTH_REQUEST_DIALOG (dialog), GTK_MESSAGE_ERROR, _("No name specified"));
+		return;
+	}
+
+	if (g_regex_match_simple ("/", name, 0, 0)) {
+		char *message;
+
+		message = g_strdup_printf (_("Invalid name. The following characters are not allowed: %s"), "/");
+		gth_request_dialog_set_info_text (GTH_REQUEST_DIALOG (dialog), GTK_MESSAGE_ERROR, message);
+
+		g_free (message);
+		g_free (name);
+
+		return;
+	}
+
+	folder = g_file_get_child_for_display_name (data->parent, name, &error);
+	if ((folder != NULL) && g_file_make_directory (folder, NULL, &error)) {
 		GList       *list;
 		GtkWidget   *folder_tree;
 		GtkTreePath *path;
 
-		list = g_list_prepend (NULL, new_folder);
+		list = g_list_prepend (NULL, folder);
 		gth_monitor_folder_changed (gth_main_get_default_monitor (),
-					    parent,
+					    data->parent,
 					    list,
 					    GTH_MONITOR_EVENT_CREATED);
 
-		folder_tree = gth_browser_get_folder_tree (browser);
-		path = gth_folder_tree_get_path (GTH_FOLDER_TREE (folder_tree), parent);
+		folder_tree = gth_browser_get_folder_tree (data->browser);
+		path = gth_folder_tree_get_path (GTH_FOLDER_TREE (folder_tree), data->parent);
 		gth_folder_tree_expand_row (GTH_FOLDER_TREE (folder_tree), path, FALSE);
 
 		gtk_tree_path_free (path);
 		g_list_free (list);
 	}
-	else
-		_gtk_error_dialog_from_gerror_show (GTK_WINDOW (browser), _("Could not create the folder"), &error);
 
-	_g_object_unref (new_folder);
-	g_free (folder_name);
+	if (error != NULL) {
+		if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_EXISTS))
+			gth_request_dialog_set_info_text (GTH_REQUEST_DIALOG (dialog), GTK_MESSAGE_ERROR, _("Name already used"));
+		else
+			gth_request_dialog_set_info_text (GTH_REQUEST_DIALOG (dialog), GTK_MESSAGE_ERROR, error->message);
+		g_clear_error (&error);
+	}
+	else {
+		new_fodler_data_free (data);
+		gtk_widget_destroy (dialog);
+	}
+
+	g_object_unref (folder);
+}
+
+
+static void
+_gth_browser_create_new_folder (GthBrowser *browser,
+				GFile      *parent)
+{
+	NewFolderData *data;
+	GtkWidget     *dialog;
+
+	data = g_new0 (NewFolderData, 1);
+	data->browser = browser;
+	data->parent = g_object_ref (parent);
+
+	dialog = gth_request_dialog_new (GTK_WINDOW (browser),
+					 GTK_DIALOG_MODAL,
+					 _("New folder"),
+					 _("Enter the folder name:"),
+					 GTK_STOCK_CANCEL,
+					 _("C_reate"));
+	g_signal_connect (dialog,
+			  "response",
+			  G_CALLBACK (new_folder_dialog_response_cb),
+			  data);
+
+	gtk_widget_show (dialog);
 }
 
 
@@ -296,6 +362,8 @@ clipboard_received_cb (GtkClipboard     *clipboard,
 	const char  *raw_data;
 	char       **clipboard_data;
 	int          i;
+	GtkTreePath *path;
+	int          position;
 	GthTask     *task;
 
 	raw_data = (const char *) gtk_selection_data_get_data (selection_data);
@@ -342,10 +410,22 @@ clipboard_received_cb (GtkClipboard     *clipboard,
 		paste_data->cut = FALSE;
 	}
 
+	position = -1;
+	path = gth_file_selection_get_last_selected (GTH_FILE_SELECTION (gth_browser_get_file_list_view (browser)));
+	if (path != NULL) {
+		int *indices;
+
+		indices = gtk_tree_path_get_indices (path);
+		if (indices != NULL)
+			position = indices[0] + 1;
+		gtk_tree_path_free (path);
+	}
+
 	task = gth_copy_task_new (paste_data->file_source,
 				  paste_data->destination,
 				  paste_data->cut,
-				  paste_data->files);
+				  paste_data->files,
+				  position);
 	gth_browser_exec_task (browser, task, FALSE);
 
 	g_object_unref (task);
@@ -426,6 +506,34 @@ gth_browser_activate_action_edit_delete (GtkAction  *action,
 }
 
 
+void
+gth_browser_activate_action_edit_rename (GtkAction  *action,
+					 GthBrowser *browser)
+{
+	GtkWidget *folder_tree;
+	GtkWidget *file_list;
+
+	folder_tree = gth_browser_get_folder_tree (browser);
+	if (gtk_widget_has_focus (folder_tree)) {
+		GthFileData *file_data;
+
+		file_data = gth_folder_tree_get_selected (GTH_FOLDER_TREE (folder_tree));
+		if ((file_data != NULL) && g_file_info_get_attribute_boolean (file_data->info, G_FILE_ATTRIBUTE_ACCESS_CAN_RENAME))
+			gth_folder_tree_start_editing (GTH_FOLDER_TREE (folder_tree), file_data->file);
+
+		_g_object_unref (file_data);
+
+		return;
+	}
+
+	file_list = gth_browser_get_file_list_view (browser);
+	if (gtk_widget_has_focus (file_list)) {
+		gth_hook_invoke ("gth-browser-file-list-rename", browser);
+		return;
+	}
+}
+
+
 void
 gth_browser_activate_action_folder_open_in_file_manager (GtkAction  *action,
 						         GthBrowser *browser)
@@ -444,7 +552,8 @@ gth_browser_activate_action_folder_open_in_file_manager (GtkAction  *action,
                             gtk_get_current_event_time (),
                             &error))
 	{
-		_gtk_error_dialog_from_gerror_run (GTK_WINDOW (browser), _("Could not open the location"), &error);
+		_gtk_error_dialog_from_gerror_run (GTK_WINDOW (browser), _("Could not open the location"), error);
+		g_clear_error (&error);
 	}
 
 	g_free (uri);
@@ -591,8 +700,10 @@ delete_folder_permanently (GtkWindow        *window,
 
 			gtk_widget_destroy (d);
 		}
-		else
-			_gtk_error_dialog_from_gerror_show (window, _("Could not delete the folder"), &error);
+		else {
+			_gtk_error_dialog_from_gerror_show (window, _("Could not delete the folder"), error);
+			g_clear_error (&error);
+		}
 	}
 	else {
 		GFile *parent;
@@ -657,8 +768,10 @@ gth_browser_activate_action_folder_trash (GtkAction  *action,
 			g_signal_connect (d, "response", G_CALLBACK (delete_folder_permanently_response_cb), delete_data);
 			gtk_widget_show (d);
 		}
-		else
-			_gtk_error_dialog_from_gerror_show (GTK_WINDOW (browser), _("Could not move the folder to the Trash"), &error);
+		else {
+			_gtk_error_dialog_from_gerror_show (GTK_WINDOW (browser), _("Could not move the folder to the Trash"), error);
+			g_clear_error (&error);
+		}
 	}
 	else {
 		GFile *parent;
@@ -750,47 +863,40 @@ copy_complete_cb (GthTask  *task,
 
 static void
 copy_files_to_folder (GthBrowser *browser,
+		      GList      *files,
 		      gboolean    move,
 		      char       *destination_uri,
 		      gboolean    view_destination)
 {
 	GthFileData      *destination_data;
 	GthFileSource    *file_source;
-	GList            *items;
-	GList            *file_list;
-	GList            *files;
 	CopyToFolderData *data;
 	GthTask          *task;
 
 	destination_data = gth_file_data_new_for_uri (destination_uri, NULL);
 	file_source = gth_main_get_file_source (destination_data->file);
 
-	items = gth_file_selection_get_selected (GTH_FILE_SELECTION (gth_browser_get_file_list_view (browser)));
-	file_list = gth_file_list_get_files (GTH_FILE_LIST (gth_browser_get_file_list (browser)), items);
-	files = gth_file_data_list_to_file_list (file_list);
-
 	data = g_new0 (CopyToFolderData, 1);
 	data->browser = g_object_ref (browser);
 	data->move = move;
 	data->destination = g_file_dup (destination_data->file);
 	data->view_destination = view_destination;
 
-	task = gth_copy_task_new (file_source, destination_data, move, files);
+	task = gth_copy_task_new (file_source, destination_data, move, files, -1);
 	g_signal_connect (task,
 			  "completed",
 			  G_CALLBACK (copy_complete_cb),
 			  data);
 	gth_browser_exec_task (browser, task, FALSE);
 
-	_g_object_list_unref (files);
-	_g_object_list_unref (file_list);
-	_gtk_tree_path_list_free (items);
+
 	g_object_unref (file_source);
 }
 
 
 static void
 copy_to_folder_dialog (GthBrowser *browser,
+		       GList      *files,
 		       gboolean    move)
 {
 	GtkWidget *dialog;
@@ -836,7 +942,7 @@ copy_to_folder_dialog (GthBrowser *browser,
 
 			/* copy / move the files */
 
-			copy_files_to_folder (browser, move, destination_uri, view_destination);
+			copy_files_to_folder (browser, files, move, destination_uri, view_destination);
 		}
 
 		g_free (destination_uri);
@@ -846,11 +952,31 @@ copy_to_folder_dialog (GthBrowser *browser,
 }
 
 
+static void
+copy_selected_files_to_folder (GthBrowser *browser,
+			       gboolean    move)
+{
+	GList *items;
+	GList *file_list;
+	GList *files;
+
+	items = gth_file_selection_get_selected (GTH_FILE_SELECTION (gth_browser_get_file_list_view (browser)));
+	file_list = gth_file_list_get_files (GTH_FILE_LIST (gth_browser_get_file_list (browser)), items);
+	files = gth_file_data_list_to_file_list (file_list);
+
+	copy_to_folder_dialog (browser, files, move);
+
+	_g_object_list_unref (files);
+	_g_object_list_unref (file_list);
+	_gtk_tree_path_list_free (items);
+}
+
+
 void
 gth_browser_activate_action_tool_copy_to_folder (GtkAction  *action,
 						 GthBrowser *browser)
 {
-	copy_to_folder_dialog (browser, FALSE);
+	copy_selected_files_to_folder (browser, FALSE);
 }
 
 
@@ -858,5 +984,39 @@ void
 gth_browser_activate_action_tool_move_to_folder (GtkAction  *action,
 						 GthBrowser *browser)
 {
-	copy_to_folder_dialog (browser, TRUE);
+	copy_selected_files_to_folder (browser, TRUE);
+}
+
+
+static void
+copy_folder_to_folder (GthBrowser *browser,
+		       gboolean    move)
+{
+	GthFileData *file_data;
+	GList       *files;
+
+	file_data = gth_browser_get_folder_popup_file_data (browser);
+	if (file_data == NULL)
+		return;
+
+	files = g_list_prepend (NULL, g_object_ref (file_data->file));
+	copy_to_folder_dialog (browser, files, move);
+
+	_g_object_list_unref (files);
+}
+
+
+void
+gth_browser_activate_action_folder_copy_to_folder (GtkAction  *action,
+						   GthBrowser *browser)
+{
+	copy_folder_to_folder (browser, FALSE);
+}
+
+
+void
+gth_browser_activate_action_folder_move_to_folder (GtkAction  *action,
+						   GthBrowser *browser)
+{
+	copy_folder_to_folder (browser, TRUE);
 }
