@@ -25,18 +25,15 @@
 #include "gtk-utils.h"
 #include "pixbuf-utils.h"
 
-#define REQUEST_ENTRY_WIDTH 220
+#define REQUEST_ENTRY_WIDTH_IN_CHARS 40
 
 
-static GtkWidget *
-create_button (const char *stock_id,
-	       const char *text)
+GtkWidget *
+_gtk_button_new_from_stock_with_text (const char *stock_id,
+				      const char *text)
 {
 	GtkWidget    *button;
-	GtkWidget    *hbox;
 	GtkWidget    *image;
-	GtkWidget    *label;
-	GtkWidget    *align;
 	const char   *label_text;
 	gboolean      text_is_stock;
 	GtkStockItem  stock_item;
@@ -46,28 +43,20 @@ create_button (const char *stock_id,
 	if (gtk_stock_lookup (text, &stock_item)) {
 		label_text = stock_item.label;
 		text_is_stock = TRUE;
-	} else {
+	}
+	else {
 		label_text = text;
 		text_is_stock = FALSE;
 	}
 
-	if (text_is_stock)
-		image = gtk_image_new_from_stock (text, GTK_ICON_SIZE_BUTTON);
-	else
-		image = gtk_image_new_from_stock (stock_id, GTK_ICON_SIZE_BUTTON);
-	label = gtk_label_new_with_mnemonic (label_text);
-	hbox = gtk_hbox_new (FALSE, 2);
-	align = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
+	image = gtk_image_new_from_stock (text_is_stock ? text : stock_id, GTK_ICON_SIZE_BUTTON);
+	gtk_button_set_image (GTK_BUTTON (button), image);
+
+	gtk_button_set_use_underline (GTK_BUTTON (button), TRUE);
+	gtk_button_set_label (GTK_BUTTON (button), label_text);
 
 	gtk_widget_set_can_default (button, TRUE);
-	gtk_label_set_mnemonic_widget (GTK_LABEL (label), GTK_WIDGET (button));
-
-	gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
-	gtk_box_pack_end (GTK_BOX (hbox), label, FALSE, FALSE, 0);
-	gtk_container_add (GTK_CONTAINER (button), align);
-	gtk_container_add (GTK_CONTAINER (align), hbox);
-
-	gtk_widget_show_all (button);
+	gtk_widget_show (button);
 
 	return button;
 }
@@ -169,110 +158,6 @@ _gtk_message_dialog_new (GtkWindow        *parent,
 }
 
 
-char *
-_gtk_request_dialog_run (GtkWindow        *parent,
-			 GtkDialogFlags    flags,
-			 const char       *message,
-			 const char       *default_value,
-			 int               max_length,
-			 const char       *no_button_text,
-			 const char       *yes_button_text)
-{
-	GtkWidget    *d;
-	GtkWidget    *label;
-	GtkWidget    *image;
-	GtkWidget    *hbox;
-	GtkWidget    *vbox;
-	GtkWidget    *entry;
-	GtkWidget    *button;
-	char         *result = NULL;
-	char         *stock_id = GTK_STOCK_DIALOG_QUESTION;
-
-	d = gtk_dialog_new_with_buttons ("", parent, flags, NULL);
-	gtk_window_set_resizable (GTK_WINDOW (d), FALSE);
-
-	gtk_container_set_border_width (GTK_CONTAINER (d), 6);
-	gtk_container_set_border_width (GTK_CONTAINER (gtk_dialog_get_content_area (GTK_DIALOG (d))), 6);
-	gtk_box_set_spacing (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (d))), 12);
-
-	/* Add label and image */
-
-	image = gtk_image_new_from_stock (stock_id, GTK_ICON_SIZE_DIALOG);
-	gtk_misc_set_alignment (GTK_MISC (image), 0.5, 0.0);
-
-	label = gtk_label_new (message);
-	gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
-	gtk_label_set_selectable (GTK_LABEL (label), FALSE);
-	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
-
-	entry = gtk_entry_new ();
-	gtk_widget_set_size_request (entry, REQUEST_ENTRY_WIDTH, -1);
-	gtk_entry_set_max_length (GTK_ENTRY (entry), max_length);
-	gtk_entry_set_text (GTK_ENTRY (entry), default_value);
-	gtk_entry_set_activates_default (GTK_ENTRY (entry), TRUE);
-
-	hbox = gtk_hbox_new (FALSE, 6);
-	vbox = gtk_vbox_new (FALSE, 6);
-
-	gtk_container_set_border_width (GTK_CONTAINER (hbox), 6);
-	gtk_box_set_spacing (GTK_BOX (hbox), 12);
-	gtk_box_set_spacing (GTK_BOX (vbox), 6);
-
-	gtk_box_pack_start (GTK_BOX (vbox), label,
-			    TRUE, TRUE, 0);
-
-	gtk_box_pack_start (GTK_BOX (vbox), entry,
-			    FALSE, FALSE, 0);
-
-	gtk_box_pack_start (GTK_BOX (hbox), image,
-			    FALSE, FALSE, 0);
-
-	gtk_box_pack_start (GTK_BOX (hbox), vbox,
-			    TRUE, TRUE, 0);
-
-	gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (d))),
-			    hbox,
-			    FALSE, FALSE, 0);
-
-	gtk_widget_show_all (hbox);
-
-	/* Add buttons */
-
-	button = create_button (GTK_STOCK_CANCEL, no_button_text);
-	gtk_dialog_add_action_widget (GTK_DIALOG (d),
-				      button,
-				      GTK_RESPONSE_CANCEL);
-
-	/**/
-
-	button = create_button (GTK_STOCK_OK, yes_button_text);
-	gtk_dialog_add_action_widget (GTK_DIALOG (d),
-				      button,
-				      GTK_RESPONSE_YES);
-
-	/**/
-
-	gtk_dialog_set_default_response (GTK_DIALOG (d),
-					 GTK_RESPONSE_YES);
-	gtk_widget_grab_focus (entry);
-
-	/* Run dialog */
-
-	if ((gtk_dialog_run (GTK_DIALOG (d)) == GTK_RESPONSE_YES) &&
-	    (strlen (gtk_entry_get_text (GTK_ENTRY (entry))) > 0) )
-		/* Normalize unicode text to "NFC" form for consistency. */
-		result = g_utf8_normalize (gtk_entry_get_text (GTK_ENTRY (entry)),
-					   -1,
-					   G_NORMALIZE_NFC);
-	else
-		result = NULL;
-
-	gtk_widget_destroy (d);
-
-	return result;
-}
-
-
 GtkWidget*
 _gtk_yesno_dialog_new (GtkWindow        *parent,
 		       GtkDialogFlags    flags,
@@ -320,14 +205,14 @@ _gtk_yesno_dialog_new (GtkWindow        *parent,
 
 	/* Add buttons */
 
-	button = create_button (GTK_STOCK_CANCEL, no_button_text);
+	button = _gtk_button_new_from_stock_with_text (GTK_STOCK_CANCEL, no_button_text);
 	gtk_dialog_add_action_widget (GTK_DIALOG (d),
 				      button,
 				      GTK_RESPONSE_CANCEL);
 
 	/**/
 
-	button = create_button (GTK_STOCK_OK, yes_button_text);
+	button = _gtk_button_new_from_stock_with_text (GTK_STOCK_OK, yes_button_text);
 	gtk_dialog_add_action_widget (GTK_DIALOG (d),
 				      button,
 				      GTK_RESPONSE_YES);
@@ -407,14 +292,14 @@ _gtk_yesno_dialog_with_checkbutton_new (GtkWindow        *parent,
 
 	/* Add buttons */
 
-	button = create_button (GTK_STOCK_CANCEL, no_button_text);
+	button = _gtk_button_new_from_stock_with_text (GTK_STOCK_CANCEL, no_button_text);
 	gtk_dialog_add_action_widget (GTK_DIALOG (d),
 				      button,
 				      GTK_RESPONSE_CANCEL);
 
 	/**/
 
-	button = create_button (GTK_STOCK_OK, yes_button_text);
+	button = _gtk_button_new_from_stock_with_text (GTK_STOCK_OK, yes_button_text);
 	gtk_dialog_add_action_widget (GTK_DIALOG (d),
 				      button,
 				      GTK_RESPONSE_YES);
@@ -546,23 +431,22 @@ _gtk_message_dialog_with_checkbutton_new (GtkWindow        *parent,
 void
 _gtk_error_dialog_from_gerror_run (GtkWindow   *parent,
 				   const char  *title,
-				   GError     **gerror)
+				   GError      *gerror)
 {
 	GtkWidget *d;
 
-	g_return_if_fail (*gerror != NULL);
+	g_return_if_fail (gerror != NULL);
 
 	d = _gtk_message_dialog_new (parent,
 				     GTK_DIALOG_DESTROY_WITH_PARENT,
 				     GTK_STOCK_DIALOG_ERROR,
 				     title,
-				     (*gerror)->message,
+				     gerror->message,
 				     GTK_STOCK_OK, GTK_RESPONSE_OK,
 				     NULL);
 	gtk_dialog_run (GTK_DIALOG (d));
 
 	gtk_widget_destroy (d);
-	g_clear_error (gerror);
 }
 
 
@@ -578,7 +462,7 @@ error_dialog_response_cb (GtkDialog *dialog,
 void
 _gtk_error_dialog_from_gerror_show (GtkWindow   *parent,
 				    const char  *title,
-				    GError     **gerror)
+				    GError      *gerror)
 {
 	GtkWidget *d;
 
@@ -586,15 +470,12 @@ _gtk_error_dialog_from_gerror_show (GtkWindow   *parent,
 				     GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
 				     GTK_STOCK_DIALOG_ERROR,
 				     title,
-				     (gerror != NULL) ? (*gerror)->message : NULL,
+				     (gerror != NULL) ? gerror->message : NULL,
 				     GTK_STOCK_OK, GTK_RESPONSE_OK,
 				     NULL);
 	g_signal_connect (d, "response", G_CALLBACK (error_dialog_response_cb), NULL);
 
 	gtk_window_present (GTK_WINDOW (d));
-
-	if (gerror != NULL)
-		g_clear_error (gerror);
 }
 
 
@@ -1107,7 +988,8 @@ _g_launch_command (GtkWidget  *parent,
 
 	app_info = g_app_info_create_from_commandline (command, name, G_APP_INFO_CREATE_SUPPORTS_URIS, &error);
 	if (app_info == NULL) {
-		_gtk_error_dialog_from_gerror_show(GTK_WINDOW (parent), _("Could not launch the application"), &error);
+		_gtk_error_dialog_from_gerror_show (GTK_WINDOW (parent), _("Could not launch the application"), error);
+		g_clear_error (&error);
 		return;
 	}
 
@@ -1115,7 +997,8 @@ _g_launch_command (GtkWidget  *parent,
 	gdk_app_launch_context_set_screen (launch_context, gtk_widget_get_screen (parent));
 
 	if (! g_app_info_launch (app_info, files, G_APP_LAUNCH_CONTEXT (launch_context), &error)) {
-		_gtk_error_dialog_from_gerror_show(GTK_WINDOW (parent), _("Could not launch the application"), &error);
+		_gtk_error_dialog_from_gerror_show (GTK_WINDOW (parent), _("Could not launch the application"), error);
+		g_clear_error (&error);
 		return;
 	}
 
